#!/usr/bin/perl

use Digest::MD5 qw(md5 md5_hex md5_base64);

print "\nCombine two Minecraft achievements/statistics files, v0.1\n";
print "verified to work for Minecraft 1.6.6\n";
print "written by Jonathan Edgecombe (see his 'statsgenerator' page (http://www.jonathanedgecombe.com/statsgenerator/)\n";
print "ported on 19/06/2011 to Perl by Patrick Min (also see http://www.patrickmin.com/minecraft)\n\n";


if (($ARGV[0] eq "") || ($ARGV[1] eq "")) {
    print "Usage: combine_stats.pl <stats file 1> <stats file 2>\n\n";
    exit;
}

&read_stats_file($ARGV[0]);
&read_stats_file($ARGV[1]);

&init_map;

&compute_checksum;

&write_stats_file;

print "done\n";
print "The file with the combined statistics is called 'combined.dat',\n which you need to rename to stats_<yourname>_unsent.dat.\nReplace <yourname> with your minecraft login name.\n\n";




sub compute_checksum
{
#    foreach $id (keys %nr) {
#	print "$id total = $nr{$id}\n";
#    }

    $c = 0;
    
    for($i=0; $i < 16; $i++) {
	$index = 5242880 + $i;
	if ($nr{$index}) {
	    $a[$c] = $index;
	    $b[$c] = $nr{$index};
	    $c++;
	}
    }  # for, achievements

    for($i=1; $i < 26; $i++) {
	&cbl($i);
    }
    &cbl(35);
    for($i=37; $i < 94; $i++) {
	&cbl($i);
    }

    for($i=256; $i < 358; $i++) {
	&cil($i);
    }
    &cil(2256);
    &cil(2257);
    for($i=1001; $i < 1005; $i++) {
	&csl($i);
    }
    for($i=2000; $i < 2009; $i++) {
	&csl($i);
    }
    for($i=2010; $i < 2012; $i++) {
	&csl($i);
    }
    for($i=2020; $i < 2026; $i++) {
	&csl($i);
    }

    &csl(1100);
    
    $session_id = $vars{"sessionid"};

    $total_string = $session_id;
    $debug_string = $session_id . "\n";
    
    for($i=0; $i < $c; $i++) {
	if ($b[$i] > 0) {
	    $total_string .= $map[$a[$i]] . "," . $b[$i] . ",";
	    $debug_string .= "$a[$i] -> $map[$a[$i]], count $b[$i]\n";
	}
    }

#    print "total string:\n$total_string\n\n";

#    print "debug string:\n$debug_string\n\n";
    
    $new_checksum = md5_hex($total_string);
    
}  # compute_checksum


    
sub write_stats_file
{
    open(STATS, "> combined.dat");

    print STATS "{\r\n";
    print STATS "  \"user\":{\r\n";
    print STATS "    \"name\":\"" . $vars{"name"} . "\",\r\n";
    print STATS "    \"sessionid\":\"" . $vars{"sessionid"} . "\"\r\n";
    print STATS "  },\r\n";
    print STATS "  \"stats-change\":[\r\n";

    for($i=0; $i < 16; $i++) {
	$index = 5242880 + $i;
	if ($nr{$index}) {
	    print STATS "    \{\"$index\":$nr{$index}\},\r\n";
	}
    }  # for, achievements

    for($i=1; $i < 26; $i++) {
	&cbl_write($i);
    }
    &cbl_write(35);
    for($i=37; $i < 94; $i++) {
	&cbl_write($i);
    }

    for($i=256; $i < 358; $i++) {
	&cil_write($i);
    }
    &cil_write(2256);
    &cil_write(2257);
    for($i=1001; $i < 1005; $i++) {
	&csl_write($i);
    }
    for($i=2000; $i < 2009; $i++) {
	&csl_write($i);
    }
    for($i=2010; $i < 2012; $i++) {
	&csl_write($i);
    }
    for($i=2020; $i < 2026; $i++) {
	&csl_write($i);
    }

    # do the 'minutes played' without a function call to avoid the trailing comma..
    $index = 1100;
    print STATS "    \{\"$index\":$nr{$index}\}\r\n";
    
    print STATS "  ],\r\n";
    print STATS "  \"checksum\":\"" . $new_checksum . "\"\r\n";
    print STATS "}\r\n";

    close STATS;

}  # write_stats_file



sub read_stats_file
{
    my($filespec) = @_;

    open(STATS, $filespec) || die "Could not open [$filespec]\n";

    while($line = <STATS>) {
	chomp $line;

	if ($line =~ /\"(.+?)\":\"?([0-9a-z]+)\"?/) {
	    $var = $1;
	    $value = $2;
	    $var =~ s/\"//g;

#	    print "[$var] = [$value]\n";

	    if ($var =~ /\d+/) {
		$nr{$var} += $value;
	    }
	    else {
		$vars{$var} = $value;
	    }
	}
    }

    close STATS;

}  # read_stats_file






sub init_map
{
    $map[1000] = "43ddd8b48469d9a8c011718aa846facb";
    $map[1001] = "aaed8c669f583cf35300f7c5d5396fce";
    $map[1002] = "d517ae73160fd8576d7687ead1c1a973";
    $map[1003] = "ac37df168bae3c69a24d7fd5bae913c7";
    $map[1004] = "6d2591e94464b8327afadbdba44978d";
    $map[1100] = "8b1df73e012e2ae34cd2d84a72a7898b";
    $map[2000] = "c9ee0d494e0524c86a577cf684f5816d";
    $map[2001] = "cd7a2836ea0b77b23a6131222f5be354";
    $map[2002] = "381aeedb2c5c5a7a9a01333e0eebc839";
    $map[2003] = "229048d1bb9e928831734cb2e5000286";
    $map[2004] = "b2c088c3c6928bfe29684a75ae1b127c";
    $map[2005] = "1ce3491a1021a5aed07c687e3f6133e6";
    $map[2006] = "6c3d114caffd8160f7d56fc8757419a";
    $map[2007] = "bcb501cfac8f7e73b455563bddbcb417";
    $map[2008] = "e1aa93d7bba48898a32810c43c6ff5c3";
    $map[2010] = "d0818779df1b967cc2f8ffc3ada631ec";
    $map[2011] = "ecf0dbe93240e84edf6bd263645e634f";
    $map[2020] = "b4a67353e11c3039ad755a3e96b4a046";
    $map[2021] = "1bb21f7731a5b3ffc6a1e2b586dae3f5";
    $map[2022] = "e58162e284df8fe339f05626eb46d835";
    $map[2023] = "739948643f5f6f685aea81c42883a480";
    $map[2024] = "799b0d35362a9574d3226c2d762b53c6";
    $map[2025] = "5ec18fbb462516a1e1a3427e0595a7d6";
    $map[16908544] = "12512904b67d2091dd516dc8eeb0cfa0";
    $map[16908545] = "a5c35cc01a263f91419d3e8e981708f1";
    $map[16908546] = "f6bdebf227e42c204066cede8cb5928c";
    $map[16908547] = "9c44e6d3ab80afa2bb52fe3499559d5e";
    $map[16908548] = "2e1799ddfdab14417c2a8e24d8c0c731";
    $map[16908549] = "3f4169a181feb5d2532efc6aa91f2ee5";
    $map[16908550] = "895725d3432cbf374b1cce4f7db28e9";
    $map[16908551] = "12892864f18d30521b6f4a6c45b65ece";
    $map[16908552] = "146f6c3458adde8e0bca04d976275f85";
    $map[16908553] = "7c40dbd9451129b4762f3ab04ce7ffb9";
    $map[16908554] = "56d763eeb6a0107c2146d7dccd686d2a";
    $map[16908555] = "9a4fff004ae419711c2a876b57cba69f";
    $map[16908556] = "170ff67dd90d8d3d8dae393037da5e9b";
    $map[16908557] = "3e65d657af8611cb3578cd8713419d4c";
    $map[16908558] = "f3cb95383f698c4e9721b5aaa5fac3a7";
    $map[16908559] = "65bd006dc163a186f2dbb127f8ec0b27";
    $map[16908560] = "8c6479d786f9117e04910153a4f47d86";
    $map[16908561] = "75a25bd69f36250789d914ac01da34c1";
    $map[16908562] = "87a28b2bf2fad838af72ce00e2fb1937";
    $map[16908563] = "249ff32d605c5ba0a07694ff4fbbf15a";
    $map[16908564] = "ba3b355eb9134bcd7522a1fc203c4c58";
    $map[16908565] = "321e1cb3657d2aca73fb9a8083d9c3bf";
    $map[16908566] = "adb182051fab2c2269df89b4f75e505e";
    $map[16908567] = "3d9ae8982f01fb6a6c56cffbe2fa074e";
    $map[16908568] = "1efffb054b2c864f5ebe180ae46891c3";
    $map[16908569] = "25c73629d0b6b46690d222cecf331ba2";
    $map[16908570] = "6a46694c996e5721b14c9da1f00fe62c";
    $map[16908571] = "5efc1c291eef23fe33f5209388b8cc33";
    $map[16908572] = "f71fc7deef6835c8efc305a00f1e8b8f";
    $map[16908573] = "15484a373254c6024d90edf3b7a82947";
    $map[16908574] = "8253f1aaa2205dcaf828903cfce46e4";
    $map[16908575] = "ab778410d1fb2c0e55fec0c7d764b17c";
    $map[16908576] = "19ad61c04ff3ae082955f72fc8771866";
    $map[16908577] = "b5d3eeff40086643a03f034b5345504d";
    $map[16908578] = "3d647bf5562e743263cbd90be36b6a1c";
    $map[16908579] = "5a9d2fb7050009785421257d5ef24416";
    $map[16908580] = "909b03e2d2eeee2e868b5aaf446e81ea";
    $map[16908581] = "e603a4cbfdfcbf4d0bfe25efb753b424";
    $map[16908582] = "e355f481f99220e50e4b2b2eb45dcc56";
    $map[16908583] = "73a25e59c2d18ae77db02928b0a70e3";
    $map[16908584] = "8f81a2fb647f817422a299829994de7b";
    $map[16908585] = "128e9978380ee9ef981d2e12d008255e";
    $map[16908586] = "fd1453f3e2a3125e7d16cfe6fd2d4722";
    $map[16908587] = "8e27b6f750bec22333a5d4108ce1e094";
    $map[16908588] = "b2169f6190bdf95236037bed2301712b";
    $map[16908589] = "536e6340e5328075ede672428d94d921";
    $map[16908590] = "cb9eede8b783b4a5d7ff7529e3fb1f44";
    $map[16908591] = "267d51d891c3c32b3a02577d38f4e242";
    $map[16908592] = "3f31e8ab260f02acca218c720fafc7b8";
    $map[16908593] = "552869a63aa3151e79e875596f31e505";
    $map[16908594] = "c5b0399e7fc64f0561a82063773e1282";
    $map[16908595] = "cfd61cdbf22133caeca528da9b53474e";
    $map[16908596] = "8d3802169313f270d5adcda790170ca2";
    $map[16908597] = "a82c4f31d6d3f5bef9aa3062945c02d6";
    $map[16908598] = "85eca649072752e9ffafdefa1287470f";
    $map[16908599] = "98ffd259abbe2e4794c00716a9d2da7";
    $map[16908600] = "263f8fab89ae0f9d695a74512443070b";
    $map[16908601] = "b4ae28e8babe7bd09b2a392780565dbe";
    $map[16908602] = "dfd84b5b35231267a47891cc8c8f48ba";
    $map[16908603] = "f6990bbd57862a238aaf68b0c2f4c998";
    $map[16908604] = "9dca837e109ae977a89ed64dd17fab10";
    $map[16908605] = "ebdb407fd6e4d5c450f73173487c3162";
    $map[16908606] = "fdf869946d814f4f2aeebe269db06e31";
    $map[16908607] = "f9ea1fa393862a86e79ffade6a2662e7";
    $map[16908608] = "60491b4f5956534afd5959e7c2411cda";
    $map[16908609] = "b78fc6d8b8b8ecb1266eb9c04c5129b";
    $map[16908610] = "add241f6dcfe6af3f096a45b96fb04e1";
    $map[16908611] = "38760b28c0d26204b1e0383ca318947a";
    $map[16908612] = "d3d8076b8b7a911b7cbbf5099f8ffd48";
    $map[16908613] = "fc78be6fb5735b715325ac499990550e";
    $map[16908614] = "a7504fcd85d6696a8880f32b97b34a60";
    $map[16908615] = "f0821b6fdc8ebbf8a6fcb37c75086b1d";
    $map[16908616] = "88b0e1b842fc88f0cbc5f38e95d68417";
    $map[16908617] = "d977f8453c349a44baa7e05c3dd81cb8";
    $map[16908618] = "26184c3c1edfa7623d5620433c0ff36";
    $map[16908619] = "8edbf7f3996852add4fcb18008cb722f";
    $map[16908620] = "39e5881696ed1141fb934593dd6bab70";
    $map[16908621] = "1e0d787240ea16af6edbbdb1ab372fe7";
    $map[16908622] = "af6ec83a0b072b4abd4194d8d43f7f02";
    $map[16908623] = "658b5f77ff892a455a7fce5dc93268ac";
    $map[16908624] = "6474336f089d541c9cec4a1df0c3b6cf";
    $map[16908625] = "72cfc6fe503df207d53119aa7d5b84b4";
    $map[16908626] = "3070c2283ca5fd0fe00f7880a4bcdf18";
    $map[16908627] = "340cef5b043dceb7073ede0be69d700d";
    $map[16908628] = "24dda1839a72d6707ed1932f6f32bd8e";
    $map[16908629] = "1c50c7d94cec18710417d513708094d1";
    $map[16908630] = "2b1f11fed636bf7892650b9adc9b8e94";
    $map[16908631] = "afd9ba61c39fa742a840fe6ab2cfb490";
    $map[16908632] = "b709157ffabe225e820218b020508173";
    $map[16908633] = "3e9f7b867952f7b2df48e156402d558";
    $map[16908634] = "ab9f22a1fbc265d19a4dd7f84f3a4585";
    $map[16908635] = "c89cf9510108d63d7d181b3f7c546094";
    $map[16908636] = "c7af3251ce44f2dafb2c192d817e9c52";
    $map[16908637] = "796f607d214012447b7c552673f95b4a";
    $map[16908638] = "ea796bd83dd7689d322085c74d0957a7";
    $map[16908639] = "ce2f6f842d68418baad5fc2ecefcb879";
    $map[16908640] = "6b3016429a5f814f3cccaff484a46831";
    $map[16908641] = "7dea9bdb69c9c5965609eab99af9f9d9";
    $map[16908642] = "3add5c50b3427182fcb7dd04fe953945";
    $map[16908643] = "4175965cd5cee134d43038ac54858b0f";
    $map[16908644] = "feae27a32b6cc95860ec46da6d37c4b0";
    $map[16908645] = "5898813ee0b2785afc9dad2524f56162";
    $map[16910544] = "adb04e128238341b88a18532f3448d13";
    $map[16910545] = "8e274d3a14b5cb87cbce019d0a3b9a23";
    $map[16974080] = "cfed396d9f54d3c04bc390a004916c52";
    $map[16974081] = "64f1b3409c0937114f77f9933a4e2b82";
    $map[16974082] = "5a6e9dc254405a6cb066472172fa09b1";
    $map[16974083] = "b23adf8b24d9a42637977c3322f02b75";
    $map[16974091] = "df1b76bca1e6a035d3e22e7c01f67bc6";
    $map[16974092] = "4bb3d702d36739ba62041c0c936cbfb6";
    $map[16974093] = "d02062bdb07f81100381ef87e6fc2922";
    $map[16974094] = "1a0d208f9f489b37b800c6eeaaa752c9";
    $map[16974095] = "34ee6807c2040168606740d0e3f46e30";
    $map[16974096] = "9b8bb8517841df03f1a894b238251068";
    $map[16974097] = "bb0b43349ffa6aa13b5d84ca70d25eb3";
    $map[16974098] = "b3b647da85fe51c6da997e28b66cb4c3";
    $map[16974099] = "d33c3d7d3fbc6880e0e03ce573644b1e";
    $map[16974100] = "17d45fba9866349c133608e9d42e4ac9";
    $map[16974101] = "74205e9529729bd0ef51c3d544c1e372";
    $map[16974102] = "edf620106dc10c10a3357436a306400d";
    $map[16974103] = "74b3a17953dca5a0cb9bb8c47b241eca";
    $map[16974107] = "1938aff98985ef5e79c7003868c1e91";
    $map[16974108] = "14b782f5d9afda571ce6c8bad868e2bd";
    $map[16974109] = "cb124718b4ecfb29a900e1e617c79544";
    $map[16974110] = "ae3c73a1546a058da74d4b8a588b59f2";
    $map[16974114] = "cbdefe9bd14088ad45141843474d2bb0";
    $map[16974115] = "641938697f84f6e7b59ebc08edc4648b";
    $map[16974116] = "74ccb1de521aa54c65ffcb906ffd1d3b";
    $map[16974117] = "fffed44e7bb844ad6c69f7eb4540cff3";
    $map[16974118] = "536b2d3a4009eed61300bf5f0af7419e";
    $map[16974122] = "8a8ea889723fc0f8de50c53339819871";
    $map[16974123] = "768b65f8e24b1e042b75cf6f576cfcd1";
    $map[16974124] = "a4b5522486920acfc2ed97e105c7831";
    $map[16974125] = "6e9b513f3c219f4276556070b36a0339";
    $map[16974126] = "942450301cd8068c9a14352c1cecc2d0";
    $map[16974127] = "c98e2b8eb252d75a2a1f90144596ada4";
    $map[16974128] = "7b13a95332dac62ad73cddf33d3bba0f";
    $map[16974129] = "1951adc5a346867490626a854788b9ad";
    $map[16974130] = "b75636944eeba77a4ce3f254460e0d49";
    $map[16974131] = "bb5bc2285c0a2e24548533f59af76788";
    $map[16974132] = "929561410b14786db52aa89b675a9759";
    $map[16974133] = "6b07bd2605f311d8134a8170ba49fd73";
    $map[16974134] = "2f97923d891e3e8c1967d53f7734ff55";
    $map[16974135] = "dea0b11c8760d28eafbe76e25df7d301";
    $map[16974136] = "586e309c00ba6f3c2ada595b742bef23";
    $map[16974137] = "29e48d74dc5062eb900d376196fd4f23";
    $map[16974138] = "fa79e8b641b225166b17ea86a9ecb174";
    $map[16974139] = "7a66328b6b976337ff5bde5c7edccaa5";
    $map[16974140] = "95559b402657cf222a87248d1e6ce0f5";
    $map[16974141] = "52843420526ced65d57513afad342515";
    $map[16974170] = "56690bacbebf8107b83e80528f40740";
    $map[16908289] = "83cfae9a11032a88e08c95b7fcef84cc";
    $map[16908291] = "39a4ea3b5ddf7f89b41faf675f7bfc0f";
    $map[16908292] = "64ec21dc8495fb180cc94f59f873efb8";
    $map[16908293] = "fda16e9858404b9545649ba21a828a6a";
    $map[16908294] = "a57157213ffc25d269ea46e45984eae0";
    $map[16908295] = "90f2ac6ac3005ad4cabcd248831f2205";
    $map[16908296] = "4adafcbbef5ace168ca03e173af10452";
    $map[16908298] = "bb2e0425d8b489fc856cab2c0828f2ff";
    $map[16908300] = "76b80613789c553593097956002a392b";
    $map[16908301] = "23bd5cc4fb97d25e432604be0958320d";
    $map[16908302] = "7b129f5775b2d4997681338012130cad";
    $map[16908303] = "4935425ac3ef59e71bca7c4c8bf7e2ea";
    $map[16908304] = "564d5c2df622b99692929bedbcd7f4b1";
    $map[16908305] = "e06bb538b6b224ce1376b47804ee2bbb";
    $map[16908306] = "f13b4a0fa66e65ed33dcbd7ead08ac46";
    $map[16908307] = "29710a54739e35ae0d34919fca091640";
    $map[16908308] = "8693b46ac89dd595f8e3efd4437e6e90";
    $map[16908309] = "3ebf10825c7d8f2e17deae70b37d1fbd";
    $map[16908310] = "354d414b2a1f29494a80b3acedb72ae2";
    $map[16908311] = "de662954670ec00899809b916abe6c60";
    $map[16908312] = "831dd7a79cd7e5f3cad86690c6724902";
    $map[16908313] = "265f19645487a858e4f53b9706cdff03";
    $map[16908314] = "2ad06f9f7e24f920fb067e2f2cc7869c";
    $map[16908315] = "99f3286cc36b9a4393b3e967878b01d4";
    $map[16908316] = "4fbc91a39f7f4814f8f79ba425e4f30a";
    $map[16908318] = "ccb82be2278587dd637ac763a1087c7";
    $map[16908323] = "8e8a2f6a87c3701e76650680e7b2f6e6";
    $map[16908325] = "fcd992a9a44518d5de6bd09c409ea7de";
    $map[16908326] = "e41e706456c10963824e032484992946";
    $map[16908327] = "662301d3344073cc69887a27a8c42f24";
    $map[16908329] = "313738e052612f868d55f47940080288";
    $map[16908330] = "79474b10f860f09be4e9183ad5f1c7b4";
    $map[16908332] = "60a3062d151151a2ff229a02d3221337";
    $map[16908333] = "833c8c420ce04aa18b9912ff05e5ced2";
    $map[16908334] = "6e0f4183ad7abbb63c5b6a1ead24214a";
    $map[16908335] = "477292deea02d72acc1087b2425c570a";
    $map[16908336] = "82cf44076f1a94ed31f3172433a80ef6";
    $map[16908337] = "84f64a9f8523d35a1cf8319f580dd578";
    $map[16908338] = "b6365376050089a025a1ed1d85a85dba";
    $map[16908339] = "4e989db8b445f22b61bd4d30daee1a05";
    $map[16908340] = "50ec890e85029d44ea96bfa31d8ed16a";
    $map[16908341] = "b4f23438454d39a742546dafd5d3cfc6";
    $map[16908342] = "5414035b686e0fb0a1ce0d535d0a0f82";
    $map[16908343] = "72d6218e86d3787422fa6b03f30ccaf3";
    $map[16908344] = "ddfe42c53aae03dd94c2d3163869ddc0";
    $map[16908345] = "b8b09ac7af1bb81e28715a3e0f0eeb17";
    $map[16908346] = "c53d28ac584d8053ec438f692a58cd1a";
    $map[16908347] = "243cf7fb2bf36e60e3a25cde83b33a5e";
    $map[16908349] = "2837ae3dabc00c4c5515bafa34a302dc";
    $map[16908351] = "4973e12438ce4c870f6ba1c158873c43";
    $map[16908352] = "37678301896aa95cc358bcd73308232c";
    $map[16908353] = "31c5e38768bda9a8a7cd1bbca7c9226d";
    $map[16908354] = "5ff1e27f9ef25fcf0a3c51b0551f7fd2";
    $map[16908355] = "60a060ee3e51167f4d87cf6d923f9ca7";
    $map[16908356] = "b24aa4a609b3483b795c4f3bf89116b7";
    $map[16908357] = "7a17a96ff758cd4ff8c0ffd070230cc7";
    $map[16908358] = "c14586711c04239a174c192d24b8ce20";
    $map[16908359] = "1f0abe2925128be5191932a1cc4a3019";
    $map[16908360] = "a9c8343cf444aa7686d9dd2b40e90a47";
    $map[16908361] = "94a02b3ef18bb35970105bf276001fe0";
    $map[16908363] = "bfa45b4169d1944debabc84b04880837";
    $map[16908365] = "a3d0bb31ba7c30209129e4366e072b05";
    $map[16908366] = "de6d35451526c85c7c5907a0adcbeff7";
    $map[16908367] = "4a0d45a9aedaa06eb574f88f580c265b";
    $map[16908368] = "56ab2510d3cb4787bdcfa9680aa7e757";
    $map[16908369] = "feef935305ae32e1972d3b017a4eb";
    $map[16908370] = "41b270e7ecb39d0b81d1b60f32c3943";
    $map[16908371] = "b159339fc55e60000efee4c55d0e1d1c";
    $map[16908372] = "74ca5387c7d373fc88e65e0623d5991f";
    $map[16908373] = "8b8ae20ea681d47cd899f74f75bde08d";
    $map[16908374] = "5cb926f4743f6ccdb4988c8b35dc97dd";
    $map[16908375] = "947a02e037fe9d31c8eb4c4a22dba58c";
    $map[16908376] = "2c2a7e8a269a770cc4afbe695e662601";
    $map[16908377] = "9a992e19651652de8b4f5ce07d1d2af0";
    $map[16908378] = "3e37efadc720d972ddc30ccae606ab00";
    $map[16908380] = "77b1802a2504a685bf8ff34a1f9de6c3";
    $map[16908381] = "45c0dbda39dca97dc634a6e89f037716";
    $map[16908383] = "ca404ded18b83fd872a5810c8a44517d";
    $map[16843027] = "ecc0eb0ba51e2c14aaa006e168eacc8c";
    $map[16842753] = "f6282e18f6f87e4cf68970579aac7219";
    $map[16843026] = "6f3d105b3e6d1624af4ce68b2a55362d";
    $map[16843025] = "aa1c8e137823bf5245f0d27c980796e7";
    $map[16843024] = "ed65803a11251872c6cf659381d0aaaa";
    $map[16843031] = "fa2f9de197813d8e32d78bd184603161";
    $map[16842757] = "1a13db89ae6fdbb152856edbe3688d95";
    $map[16843030] = "46bd8fadf2f12988d808e69dcfc138cb";
    $map[16843029] = "bf398d8ec9fef1d04c67fbf474fda790";
    $map[16843028] = "69ee29bc3bd715b3203d888ca2a88cc0";
    $map[16843035] = "2f79ed9030fe0339dec2eb43f56fa369";
    $map[16843034] = "e2f097a36cdf87a1b211d27eb31dea32";
    $map[16843033] = "c02e9379a6624b40fef669494a7e7183";
    $map[16843032] = "af44f7fe9a4366c67479a44b24ae80b";
    $map[16843038] = "501ead1cb03222808aa09b376c9d5cac";
    $map[16843037] = "a2d01853631a9b52006ad32497ddc865";
    $map[16843036] = "6e381ac09bfc75ef7f71e02f3b99f279";
    $map[16843010] = "bcfe734d49949142a5073766188213dc";
    $map[16843011] = "88ea7e11c57b09139afe92169c443e20";
    $map[16843008] = "91f14375672f1b2003202ec7682771cb";
    $map[16843009] = "f5007e4886946d1eb603979a35726d69";
    $map[16843014] = "b0bde3daab8a1cc6354cd86c65db90e2";
    $map[16842772] = "b59c880de07e668eaba0888fc4de229a";
    $map[16843015] = "7184233667abf34d2ee3eea20fb9f82b";
    $map[16842775] = "bcc06ff4c56de9b4a1557d3c47ec1783";
    $map[16842774] = "6f2eab110a33c15cb17238708244db4d";
    $map[16843013] = "3fe45d065187550d481cc745780c29c2";
    $map[16843018] = "7bddb0161d99059f0a6c1c09d0b35b34";
    $map[16842777] = "fdc779803765398f0c782a1fd55a9d86";
    $map[16842776] = "6b6c09ee89a933288c37f55d2187eddd";
    $map[16843019] = "87545673867c5cf2f97ea6cc3e0d3595";
    $map[16843016] = "e2288c9f60d09d5f2882175abf161bd";
    $map[16842779] = "9667a31366f5dcf740d64b775402c64";
    $map[16843017] = "e3108d8ee8530e50d7b425de217a4585";
    $map[16843022] = "847f1b6f38ebf442a1d34cac0fd6080e";
    $map[16843023] = "5d01d19f8441e4d5dd5c41266f480ffc";
    $map[16842780] = "b02509b360e6f14eac4fb03f55c11fed";
    $map[16843020] = "90e2ab194dab9f482eca64c651ea41e9";
    $map[16843021] = "58d3e73bd29c14b6ec97d4187cee5500";
    $map[16843057] = "bf533b7d78ccc350cfd2f41a8e1f0822";
    $map[16842787] = "e49146643faf3308ca7246d82fc5b2bb";
    $map[16843056] = "d8e023c15aa80df2445c782c6b2179da";
    $map[16843059] = "c649596d99d96be696332cb08b4a33f2";
    $map[16843058] = "d129fce211dfb56e80195a11f39ab1e9";
    $map[16843061] = "441d86175b74884ef3f672066ae52c90";
    $map[16843060] = "cf6d7dfef99b200a4551cd65467fb9fc";
    $map[16843063] = "f50f8b7fafde6d640ced3ec4d797f807";
    $map[16843062] = "d22de0ebc290280b2699b5cb77568fcb";
    $map[16842794] = "d347d92a886b9d445d7d60b090a5bcda";
    $map[16843065] = "a393477443f20a1a8a511c390024f963";
    $map[16843064] = "e6f28eea0ebbad0d18d23ea3c15f2102";
    $map[16843067] = "d6efe6a63f529d675417c2092546e144";
    $map[16842793] = "d5ac3bcc56cbc370ae3b172dd6ed5df4";
    $map[16843066] = "935aa84de286e8bced6a127b69347997";
    $map[16842798] = "377a21bcb39dcfcb4a0902a7555a984f";
    $map[16843069] = "63cc6da6165cc068740827890b4460ed";
    $map[16843068] = "6dda3129884399b67b935bb4e51ad34f";
    $map[16842799] = "d4831a785c1c65ad2054bb1232babf0c";
    $map[16842796] = "48be99a37d3a885d38657a50d6f77365";
    $map[16842797] = "8da03c2f611c7f0654aa71096832b5c";
    $map[16842802] = "9b2a96570a99f6677e299d8215b1eecd";
    $map[16843042] = "322d7c02f205806126f1529564ab3cd0";
    $map[16843043] = "660acf0ca76261ec49deb7a7bd072f8c";
    $map[16843044] = "d8dd28acc9e4431ff8eb481c1d8d9832";
    $map[16842806] = "919bc6715eddf74c0517372ead95cdcb";
    $map[16843045] = "72ae59db2ef5c472f671b31f1c6f07ce";
    $map[16842805] = "9e359ce06b5603c820bd749592beb4a7";
    $map[16843046] = "cee09c2303e44dfbbaadbf167cdc503c";
    $map[16842810] = "8ecd9ec7607c958cdf2e252f516fe3d8";
    $map[16843049] = "47171b2af8bb14c3816f52f79579b46d";
    $map[16842809] = "fef5bbbc588422579bf3de8a49c73e18";
    $map[16843050] = "6b29998dbe690122afdf6da71e905009";
    $map[16843051] = "5f9f017aa89719c7a282aa222c65baf";
    $map[16843052] = "216a61701714cb3f04e23a74a1336864";
    $map[16843053] = "e146704e3fe30e6821352ceb3b090184";
    $map[16842813] = "899942760d5a83d94cf58a536be6171e";
    $map[16843054] = "5aa0a467c7451f88f9c6056e8c7427cc";
    $map[16843055] = "6881fc74a0574433e9e0f05f7cae53f7";
    $map[16843095] = "50f6c2ffdc69bea12cd7b58c69824be1";
    $map[16842821] = "42b4814e4e61191b61a8d3f7c6041938";
    $map[16843094] = "356c3f0d5b88d0e2077d85168eac6306";
    $map[16842822] = "d69011621d777519c5ab6a68da8959";
    $map[16843092] = "d84a39f3d30a72b13640e3afb7ea2018";
    $map[16843091] = "37603e187bd7ad9dceccb8ef2b6b5335";
    $map[16842817] = "bad6f1072232c28353aabf42142cbd26";
    $map[16842818] = "bd0c4079c99e529e55abef5ac44dcdf8";
    $map[16843088] = "5e5d08b1780bfd69747216c24b7aee92";
    $map[16842819] = "ae608e1ca5ca083163d92974d7c66b06";
    $map[16843103] = "d4d1b946336039e515c3e4a392694223";
    $map[16842828] = "6a15bcf38bcf78021f9416ac395cac85";
    $map[16843102] = "603020acbbdc6e8765c2e2cd589f7933";
    $map[16842829] = "41cce73f9ce59993afa3517becd4bd3c";
    $map[16842824] = "2368ffe803beb1d06772e96eeffd9cc9";
    $map[16843099] = "c373e78822d440c926edf7f1b700d4c5";
    $map[16843098] = "503a4fb49cbfebe1fb71ef185c8ced56";
    $map[16843097] = "4e72dc4d35b34147ce69d691aa2937e3";
    $map[16842837] = "4c903ddf3b9cd49a952a3889fc2c5573";
    $map[16842836] = "a304f875b846d65498366479fbf34568";
    $map[16843076] = "1b11a34d5c6cee280c727938ee96054b";
    $map[16843077] = "ff170af78a0a674445f16e24394546c0";
    $map[16843074] = "ba8bd10757ab9a4446f024952bf3f1e3";
    $map[16843075] = "b34ec8f0f5f92acf230925c89eacec65";
    $map[16842832] = "3b10180dc99ab99568b51010d89c655c";
    $map[16843072] = "d752bd5dd907feec915d036d76707ee5";
    $map[16843073] = "168e43895b105169862fa2e3c205529b";
    $map[16842834] = "58efc828918aa006d5af09e32fe7799b";
    $map[16843085] = "b30c099494e63d26f755d58ba109136a";
    $map[16842841] = "2c471142bddf9a982a48e0ecb52cbb8";
    $map[16843082] = "5574b9699d71e02f3efb48f3712da8bf";
    $map[16843080] = "4945e39d1260ff273c49d39dfc58c2a8";
    $map[16842843] = "bfc9dce4c1ccef959c57360fb9f70205";
    $map[16843108] = "4e225dd49c0231b617a15ded1ba6cb0a";
    $map[16843109] = "35715377a038019c56925b2e661cfdab";
    $map[16843105] = "3b91182484dc677a0bf27263e612d818";
    $map[16843106] = "affab7221afa03df4b7306b9ce50be7f";
    $map[16843107] = "e3710bc197d17e0dc78ae3f607c46048";
    $map[16777217] = "9012132a1d0f770eca67e62a68247137";
    $map[16777219] = "58fc65723143afeb33f742c17e028c94";
    $map[16777220] = "1d5e2088f7137e304366873dd829ca98";
    $map[16777221] = "66a18fa3f554756edc12bf1a08586bc8";
    $map[16777222] = "4007295f7bdae0b78a6e47a9d64521";
    $map[16777228] = "c154160ce7b52d4bdc220fc954edf796";
    $map[16777229] = "2400808c6f442f2663b6869e4930c879";
    $map[16777230] = "7ae3b31e8e0f2f22f50fbf224e1fe4f7";
    $map[16777231] = "fe5698e183b10037e38f979a411489b8";
    $map[16777232] = "aeae00c7b4a95ff05423c1629bf3ad7e";
    $map[16777233] = "72153ea3d60930cebf4c0d37fac89102";
    $map[16777235] = "f517ab768fbb35e275440b74b6631bfc";
    $map[16777236] = "623df9298630687873579ea16bb97e7b";
    $map[16777237] = "6984cb9e6a322728be736860b97b971a";
    $map[16777238] = "6947378f4e2abb24b47e2b943945b16f";
    $map[16777239] = "55a585682bf0569a60a71a66f031ca2";
    $map[16777240] = "f653295b0c48244dc2aa68fc5b94fa29";
    $map[16777241] = "c213fb7ee968403b24ff6ede98686add";
    $map[16777243] = "afd641da6f3834b319b761ba6d72ff74";
    $map[16777244] = "8d822c710a221b8a407b2def97885470";
    $map[16777246] = "9c99187cc6fd9c958417b4736772e4d3";
    $map[16777251] = "6418210d69669c4a602148ed42ab2ecd";
    $map[16777253] = "afe51ea56dc8c831f9b0b5ae02bac702";
    $map[16777254] = "95812c1d4021558b6a7f62521228c52a";
    $map[16777255] = "3a1252acb6fe968751d2b61d57557e09";
    $map[16777257] = "fd7caeeb5d16756028f19ba016ed1970";
    $map[16777258] = "6339b8043c05d19c6b15135eade95f94";
    $map[16777260] = "aa1ed939a8b275df9dc5474bc0477a33";
    $map[16777261] = "b631dcf219d3add3560f243ff6cceb75";
    $map[16777262] = "91eca0c33ac53fb155feb15db48c43de";
    $map[16777263] = "b681c19d7812c281b4b2feabb88697f4";
    $map[16777264] = "a2b07ae4f37a65fe1f5992826b578b02";
    $map[16777265] = "af5255a4dcb486e2ab11669fd3de8441";
    $map[16777266] = "8a4b2b706f219e52a261e342b47a3964";
    $map[16777269] = "921cb54cfb47ca9eca0edcf8e660e3cd";
    $map[16777270] = "b79b3a522d1e2f865eb1e9aeb63dc66a";
    $map[16777272] = "36e1c1dd8cbb267488e1669811319e7e";
    $map[16777273] = "c84a7d710228024c2e100a79a2c30e55";
    $map[16777274] = "a1674e3020689bae5680f29f937e5424";
    $map[16777277] = "46b34dd723f7efeb38f670925ee3824f";
    $map[16777281] = "da71e3e4f9a25c6621d2f0b40ffd1c72";
    $map[16777282] = "388109bb9b9b49c1f5bb604abee91671";
    $map[16777283] = "eb6912fd84b2d57ed3433acb3f85d3e6";
    $map[16777285] = "619967351702f1d570d4d0195bbbcc1e";
    $map[16777286] = "ebff74d54387fb154d5b467e01d92b64";
    $map[16777288] = "f8154dd7c9c1d397dd64eeb6569d7d34";
    $map[16777289] = "971cc6f827398c6c4755b310c6795672";
    $map[16777291] = "bb952ff9523f248ddaae56317e91a509";
    $map[16777293] = "e5c24577cf35a31bd217d206b88a7499";
    $map[16777294] = "f4c170867c75982e604358da611b26e3";
    $map[16777295] = "922200512d058ecc0ef19a0efcdb51bf";
    $map[16777296] = "39283c3dbafb10b2a732f9d6ef3e08b6";
    $map[16777297] = "3603ae881c41c957a89aaa30990c0040";
    $map[16777298] = "fbf9e47600457d091060b0f877458a83";
    $map[16777300] = "2b0d53bafa91c68bf59b5863354bd7ae";
    $map[16777301] = "50b99fabae858e971eb02ed5146fd63a";
    $map[16777302] = "7a32eac396b4cf3930ba9ca7b5d8dbae";
    $map[16777303] = "115f1e956219026bb9cca0bcafb2b11b";
    $map[16777304] = "afb2f032a96d859ca5f24d8ff129bd64";
    $map[16777305] = "372b3a0fd7c09f64f3cfcab89f03a9a0";
    $map[16777306] = "e4b6233ac91337dcc3940e309d21c3c";
    $map[16777311] = "d290638555ce76ce3053c70fdacf7d55";
    $map[5242880] = "8099ff561e194072c9086dea38757a89";
    $map[5242881] = "90b3d9a90a8cf3e21527fc8fe3b43c49";
    $map[5242882] = "c9806dd45be8ebed3e4ab94f66e65212";
    $map[5242883] = "f382ded5f9a4299e3879dea4f7fe1c50";
    $map[5242884] = "27af08a994f76a2a08ec8671d14fdcc2";
    $map[5242885] = "9969ce4355ae7338470461f48f4e5ef5";
    $map[5242886] = "39208e0b0f070629cad494da5fb95f0d";
    $map[5242887] = "678e63329d3d813f22e0f15006a93768";
    $map[5242888] = "694ffe9efa49193780d4b757115c9064";
    $map[5242889] = "9080a51ccefd23f9924bae9a854e7b49";
    $map[5242890] = "5f1d51ccd1b4adf141707907233f8379";
    $map[5242891] = "73df45f45e0b7484ef51441ad0e50604";
    $map[5242892] = "c9e52234c475354483dc3e9e6386af61";
    $map[5242893] = "3c3ee1df989ecf5de70a5673acfa33b8";
    $map[5242894] = "468cc8b546e1586406b78ad733976f5f";
    $map[5242895] = "e546a7afaaa6c6729e3b062fc4ace4ae";

}  # init_map



sub cbl
{
    my($i) = @_;

    $a[$c] = 16777216 + $i;
    $b[$c] = $nr{$a[$c]};
    if ($b[$c] eq "") { $b[$c] = '0'; }
    $c++;

    $a[$c] = 16842752 + $i;
    $b[$c] = $nr{$a[$c]};
    if ($b[$c] eq "") { $b[$c] = '0'; }
    $c++;
    
    $a[$c] = 16908288 + $i;
    $b[$c] = $nr{$a[$c]};
    if ($b[$c] eq "") { $b[$c] = '0'; }
    $c++;

}  # cbl



sub cbl_write
{
    my($i) = @_;
    
    $index = 16777216 + $i;
    if ($nr{$index}) {
	print STATS "    \{\"$index\":$nr{$index}\},\r\n";
    }
    
    $index = 16842752 + $i;
    if ($nr{$index}) {
	print STATS "    \{\"$index\":$nr{$index}\},\r\n";
    }
    
    $index = 16908288 + $i;
    if ($nr{$index}) {
	print STATS "    \{\"$index\":$nr{$index}\},\r\n";
    }

}  # cbl_write



sub cil
{
    my($i) = @_;

    $a[$c] = 16973824 + $i;
    $b[$c] = $nr{$a[$c]};
    if ($b[$c] eq "") { $b[$c] = '0'; }
    $c++;

    $a[$c] = 16842752 + $i;
    $b[$c] = $nr{$a[$c]};
    if ($b[$c] eq "") { $b[$c] = '0'; }
    $c++;
    
    $a[$c] = 16908288 + $i;
    $b[$c] = $nr{$a[$c]};
    if ($b[$c] eq "") { $b[$c] = '0'; }
    $c++;

}  # cil



sub cil_write
{
    my($i) = @_;
    
    $index = 16973824 + $i;
    if ($nr{$index}) {
	print STATS "    \{\"$index\":$nr{$index}\},\r\n";
    }
    
    $index = 16842752 + $i;
    if ($nr{$index}) {
	print STATS "    \{\"$index\":$nr{$index}\},\r\n";
    }
    
    $index = 16908288 + $i;
    if ($nr{$index}) {
	print STATS "    \{\"$index\":$nr{$index}\},\r\n";
    }

}  # cil_write



sub csl
{
    my($i) = @_;

    $a[$c] = $i;
    $b[$c] = $nr{$a[$c]};
    $c++;

}  # csl



sub csl_write
{
    my($i) = @_;

    $index = $i;
    if ($nr{$index}) {
	print STATS "    \{\"$index\":$nr{$index}\},\r\n";
    }

}  # csl_write


